﻿namespace Phoenix.Model.Dal.Repositories
{
    using System.Collections.Generic;
    using System.Linq;

    using NHibernate.Linq;
    using NHibernate.Mapping;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;

    /// <summary>
    /// Repository to store Drugs
    /// </summary>
    public class DrugRepository : Repository<Drug>, IDrugRepository
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DrugRepository"/> class.
        /// </summary>
        public DrugRepository()
            : base("Drug", "Title")
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Determines whether the specified item is referenced.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        /// 	<c>true</c> if the specified item is referenced; otherwise, <c>false</c>.
        /// </returns>
        public override bool IsReferenced(Drug item)
        {
            return (from Prescription p in this.Session.Linq<Prescription>()
                    where p.Drugs.Contains(item)
                    select p).Count() > 0;
        }

        /// <summary>
        /// Searches all the drugs with the specified name.
        /// </summary>
        /// <param name="withName">Name of the drug to find.</param>
        /// <returns></returns>
        public IList<Drug> Search(string withName)
        {
            if (string.IsNullOrEmpty(withName)) return new List<Drug>();
            else if (withName == "*") return this.GetAllRows();
            else return (from Drug drug in this.Session.Linq<Drug>()
                         where drug.Title.Contains(withName)
                         select drug).ToList();
        }

        /// <summary>
        /// Searches thedrugs with specified title and type.
        /// If one of the argument is null or empty the criterion
        /// won't be used for the search.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="type">The type.</param>
        /// <returns></returns>
        public IList<Drug> Search(string title, DrugType type)
        {
            if (string.IsNullOrEmpty(title) || type == null) return new List<Drug>();
            else return (from drug in this.Session.Linq<Drug>()
                         where drug.Title == title
                         && drug.Type == type
                         select drug).ToList();
        }

        /// <summary>
        /// Searches the specified type.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <returns></returns>
        public IList<Drug> Search(DrugType type)
        {
            if (type == null) return new List<Drug>();
            else return (from drug in this.Session.Linq<Drug>()
                         where drug.Type == type
                         select drug).ToList();
        }

        #endregion Methods
    }
}