﻿namespace Phoenix.Plugins.Core
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Reflection;
    using System.Xml;

    using ICSharpCode.SharpZipLib.Core;
    using ICSharpCode.SharpZipLib.Zip;

    using Phoenix.Plugins.Core.Exceptions;
    using Phoenix.Presenter.Interfaces;

    public class PluginManager
    {
        #region Fields

        public static bool isLoaded = false;

        private DirectoryInfo pluginRepository;

        #endregion Fields

        #region Constructors

        public PluginManager(string path)
        {
            this.pluginRepository = new DirectoryInfo(path);
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the path of the plugin repository.
        /// </summary>
        /// <value>The repository path.</value>
        public string RepositoryPath
        {
            get
            {
                return this.pluginRepository.FullName;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Gets the plugins info.
        /// </summary>
        /// <returns></returns>
        public PluginInfo[] GetPluginsInfo()
        {
            List<PluginInfo> pluginsInfo = new List<PluginInfo>();

            this.ForeachFileInRepository(file =>
            {
                if (file.Extension != ".xml") return;

                var document = new XmlDocument();
                document.Load(file.FullName);

                pluginsInfo.Add(this.GetInfo(document));
            });

            return pluginsInfo.ToArray();
        }

        /// <summary>
        /// Get a list of not configured session plugin
        /// </summary>
        public IManager[] GetSessionPlugins()
        {
            List<IManager> result = new List<IManager>();
            this.ForeachFileInRepository(file =>
            {
                if (file.Extension.ToLower() != ".dll") return;
                var asm = Assembly.LoadFile(file.FullName);
                var managers = this.GetSessionPluginsIn(asm);
                result.AddRange(managers);
            });
            return result.ToArray();
        }

        /// <summary>
        /// Installs the plugin with the specified path.
        /// </summary>
        /// <param name="pluginPath">The plugin path.</param>
        public void Install(string pluginPath)
        {
            var zipFile = this.GetZipFile(pluginPath);

            this.Check(zipFile);

            foreach (ZipEntry current in zipFile)
            {
                if (current.IsFile) this.CopyToOutput(zipFile, current);
            }
        }

        /// <summary>
        /// Load and configure all the application plugins.
        /// </summary>
        public void LoadApplicationPlugins(IPluginHost host)
        {
            try
            {
                //if (isLoaded) return;

                this.ForeachFileInRepository(file =>
                {
                    if (file.Extension.ToLower() != ".dll") return;

                    var asm = Assembly.LoadFile(file.FullName);
                    this.LoadApplicationPlugins(asm, host);
                });
                isLoaded = true;
            }
            catch (Exception ex)
            {
                throw new PluginException("Impossible to load a plugin: " + ex.Message, ex);
            }
        }

        /// <summary>
        /// Uninstalls the plugin with the specified info.
        /// </summary>
        /// <param name="pluginInfo">The plugin info.</param>
        public void Uninstall(PluginInfo pluginInfo)
        {
            var directory = Path.Combine(this.RepositoryPath, pluginInfo.Directory);
            Directory.Delete(directory, true);
        }

        private void Check(ZipFile zipFile)
        {
            if (zipFile.Count != 3) throw new InvalidPluginException("The plugin installation file doesn't contain the expected data");

            foreach (ZipEntry current in zipFile)
            {
                if (!current.IsFile) continue;
                var fileInfo = new FileInfo(current.Name);
                if (fileInfo.Extension == ".xml")
                {

                    XmlDocument document = new XmlDocument();
                    document.Load(zipFile.GetInputStream(current));
                    if (!this.IsValid(document)) throw new InvalidXmPluginException();
                }
            }
        }

        private void CopyToOutput(ZipFile zipFile, ZipEntry current)
        {
            Stream zipStream = zipFile.GetInputStream(current);
            String fullOutputPath = Path.Combine(pluginRepository.FullName, current.Name);
            string directoryName = Path.GetDirectoryName(fullOutputPath);

            if (directoryName.Length > 0) Directory.CreateDirectory(directoryName);

            using (FileStream streamWriter = File.Create(fullOutputPath))
            {
                byte[] buffer = new byte[4096];		// 4K is optimum
                StreamUtils.Copy(zipStream, streamWriter, buffer);
            }
        }

        private void ForeachFileInRepository(Action<FileInfo> action)
        {
            foreach (var directory in pluginRepository.GetDirectories())
            {
                foreach (var file in directory.GetFiles())
                {
                    action(file);
                }
            }
        }

        private Type[] GetApplicationPluginsIn(Type[] types)
        {
            List<Type> result = new List<Type>();
            foreach (var type in types)
            {
                if (type.GetInterface(typeof(IApplicationPlugin).Name) != null)
                    result.Add(type);
            }
            return result.ToArray();
        }

        private PluginInfo GetInfo(XmlDocument document)
        {
            try
            {
                PluginInfo plugin = new PluginInfo();
                plugin.Name = document.GetElementsByTagName("name")[0].InnerText;
                plugin.Version = document.GetElementsByTagName("version")[0].InnerText;
                plugin.Description = document.GetElementsByTagName("description")[0].InnerText;
                plugin.ValidFrom = document.GetElementsByTagName("valid")[0].Attributes["from"].Value;
                plugin.ValidTo = document.GetElementsByTagName("valid")[0].Attributes["to"].Value;
                plugin.Directory = document.GetElementsByTagName("directory")[0].InnerText;
                return plugin;
            }
            catch (Exception ex)
            {
                throw new InvalidXmPluginException(ex);
            }
        }

        private Type[] GetSessionPluginsIn(Type[] types)
        {
            List<Type> result = new List<Type>();
            foreach (var type in types)
            {
                if (type.GetInterface(typeof(IManager).Name) != null)
                    result.Add(type);
            }
            return result.ToArray();
        }

        private IManager[] GetSessionPluginsIn(Assembly assembly)
        {
            List<IManager> result = new List<IManager>();
            var managers = this.GetSessionPluginsIn(assembly.GetTypes());

            foreach (var manager in managers)
            {
                var instance = Activator.CreateInstance(manager) as IManager;
                if (instance != null) result.Add(instance);
            }
            return result.ToArray();
        }

        private ZipFile GetZipFile(string path)
        {
            FileStream stream = File.OpenRead(path);
            ZipFile zipFile = new ZipFile(stream);
            return zipFile;
        }

        private bool IsValid(XmlDocument document)
        {
            if (document.GetElementsByTagName("name").Count == 0) return false;
            else if (document.GetElementsByTagName("version").Count == 0) return false;
            else if (document.GetElementsByTagName("description").Count == 0) return false;
            else if (document.GetElementsByTagName("valid").Count == 0) return false;
            else if (document.GetElementsByTagName("valid").Count == 0) return false;
            else if (document.GetElementsByTagName("file").Count == 0) return false;
            else return true;
        }

        private void LoadApplicationPlugins(Assembly assembly, IPluginHost host)
        {
            var plugins = this.GetApplicationPluginsIn(assembly.GetTypes());

            foreach (var plugin in plugins)
            {
                var instance = Activator.CreateInstance(plugin) as IApplicationPlugin;
                if (instance != null) instance.Start(host);
            }
        }

        #endregion Methods

        #region Nested Types

        public struct PluginInfo
        {
            #region Fields

            public string Description;
            public string Directory;
            public string Name;
            public string ValidFrom;
            public string ValidTo;
            public string Version;

            #endregion Fields
        }

        #endregion Nested Types
    }
}