﻿namespace Phoenix.View.Configuration
{
    using System.ComponentModel;
    using System.Windows.Forms;

    using Phoenix.View.Properties;

    /// <summary>
    /// Phoenix's shortcutsNode' management tool.
    /// </summary>
    public sealed class ShortcutConfiguration : IConfigurationMemento
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ShortcutConfiguration"/> class.
        /// </summary>
        public ShortcutConfiguration()
        {
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the file/configuration shortcut.
        /// </summary>
        /// <value>The file/configuration.</value>
        [Category("File")]
        [DisplayName("Configuration")]
        public Keys File_Configuration
        {
            get { return (Keys)Settings.Default.Shortcut_File_Configuration; }
            set { Settings.Default.Shortcut_File_Configuration = (int)value; }
        }

        /// <summary>
        /// Gets or sets the file/quit shortcut.
        /// </summary>
        /// <value>The file/quit.</value>
        [Category("File")]
        [DisplayName("Quit")]
        public Keys File_Quit
        {
            get { return (Keys)Settings.Default.Shortcut_File_Quit; }
            set { Settings.Default.Shortcut_File_Quit = (int)value; }
        }

        /// <summary>
        /// Gets or sets the Help/About menu shortcut.
        /// </summary>
        /// <value>The Help/About menu.</value>
        [Category("Help")]
        [DisplayName("Help About Menu")]
        public Keys Help_Help
        {
            get
            {
                return (Keys)Settings.Default.Shortcut_Help_Help;
            }
            set
            {
                Settings.Default.Shortcut_Help_Help = (int)value;
            }
        }

        /// <summary>
        /// Gets or sets the session/close session shortcut.
        /// </summary>
        /// <value>The session/close session.</value>
        [Category("Session")]
        [DisplayName("Close Session")]
        public Keys Session_CloseSession
        {
            get { return (Keys)Settings.Default.Shortcut_Session_CloseSession; }
            set { Settings.Default.Shortcut_Session_CloseSession = (int)value; }
        }

        /// <summary>
        /// Gets or sets the session/delete patient shortcut.
        /// </summary>
        /// <value>The session_ delete patient.</value>
        [Category("Session")]
        [DisplayName("Delete Patient")]
        public Keys Session_DeletePatient
        {
            get { return (Keys)Settings.Default.Shortcut_Session_DeletePatient; }
            set { Settings.Default.Shortcut_Session_DeletePatient = (int)value; }
        }

        /// <summary>
        /// Gets or sets the session/quick insert session shortcut.
        /// </summary>
        /// <value>The session/quick insert session.</value>
        [Category("Session")]
        [DisplayName("Insert Patient")]
        public Keys Session_InsertPatient
        {
            get { return (Keys)Settings.Default.Shortcut_Session_InsertPatient; }
            set { Settings.Default.Shortcut_Session_InsertPatient = (int)value; }
        }

        /// <summary>
        /// Gets or sets the session/open session shortcut.
        /// </summary>
        /// <value>The session/open session.</value>
        [Category("Session")]
        [DisplayName("Open Session")]
        public Keys Session_OpenSession
        {
            get { return (Keys)Settings.Default.Shortcut_Session_OpenSession; }
            set { Settings.Default.Shortcut_Session_OpenSession = (int)value; }
        }

        /// <summary>
        /// Gets or sets the view/log menu shortcut.
        /// </summary>
        /// <value>The view/log menu.</value>
        [Category("View")]
        [DisplayName("View Log")]
        public Keys View_Log
        {
            get
            {
                return (Keys)Settings.Default.Shortcut_View_Log;
            }
            set
            {
                Settings.Default.Shortcut_View_Log = (int)value;
            }
        }

        /// <summary>
        /// Gets or sets the view/navigation menu shortcut.
        /// </summary>
        /// <value>The view/navigation menu.</value>
        [Category("View")]
        [DisplayName("View Navigation Menu")]
        public Keys View_NavigationMenu
        {
            get { return (Keys)Settings.Default.Shortcut_View_NavigationMenu; }
            set { Settings.Default.Shortcut_View_NavigationMenu = (int)value; }
        }

        /// <summary>
        /// Gets the state of this instance.
        /// </summary>
        /// <value>The state.</value>
        private Memento State
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Commits the changes done in the config file.
        /// </summary>
        public void CommitChanges()
        {
            Settings.Default.Save();
            Settings.Default.Reload();
        }

        /// <summary>
        /// Loads this instance.
        /// </summary>
        public void Load()
        {
            Settings settings = Settings.Default;
            this.File_Configuration = (Keys)settings.Shortcut_File_Configuration;
            this.File_Quit = (Keys)settings.Shortcut_File_Quit;

            this.Session_CloseSession = (Keys)settings.Shortcut_Session_CloseSession;
            this.Session_DeletePatient = (Keys)settings.Shortcut_Session_DeletePatient;
            this.Session_InsertPatient = (Keys)settings.Shortcut_Session_InsertPatient;
            this.Session_OpenSession = (Keys)settings.Shortcut_Session_OpenSession;
            this.Session_InsertPatient = (Keys)settings.Shortcut_Session_InsertPatient;

            this.View_Log = (Keys)settings.Shortcut_View_Log;
            this.View_NavigationMenu = (Keys)settings.Shortcut_View_NavigationMenu;

            this.SaveState();
        }

        /// <summary>
        /// Rollbacks this instance to the previous saved state.
        /// </summary>
        public void Rollback()
        {
            this.File_Configuration = this.State.File_Configuration;
            this.File_Quit = this.State.File_Quit;
            this.View_NavigationMenu = this.State.View_NavigationMenu;
            this.Session_CloseSession = this.State.Session_CloseSession;
            this.Session_OpenSession = this.State.Session_OpenSession;
            this.Session_InsertPatient = this.State.Session_QuickInsertPatient;
        }

        /// <summary>
        /// Saves the state of this instance.
        /// </summary>
        public void SaveState()
        {
            Memento state = new Memento();
            state.File_Configuration = this.File_Configuration;
            state.File_Quit = this.File_Quit;
            state.View_NavigationMenu = this.View_NavigationMenu;
            state.Session_CloseSession = this.Session_CloseSession;
            state.Session_OpenSession = this.Session_OpenSession;
            state.Session_QuickInsertPatient = this.Session_InsertPatient;
            state.Session_InsertPatient = this.Session_InsertPatient;
            this.State = state;
        }

        #endregion Methods

        #region Nested Types

        /// <summary>
        /// Used to save the state of this instance.
        /// </summary>
        internal class Memento
        {
            #region Properties

            /// <summary>
            /// Gets or sets the file/configuration shortcut.
            /// </summary>
            /// <value>The file/configuration.</value>
            public Keys File_Configuration
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the file/quit shortcut.
            /// </summary>
            /// <value>The file/quit.</value>
            public Keys File_Quit
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the session/close session shortcut.
            /// </summary>
            /// <value>The session/close session.</value>
            public Keys Session_CloseSession
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the session/insert patient.
            /// </summary>
            /// <value>The session/insert patient.</value>
            public Keys Session_InsertPatient
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the session/open session shortcut.
            /// </summary>
            /// <value>The session/open session.</value>
            public Keys Session_OpenSession
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the session/quick insert session shortcut.
            /// </summary>
            /// <value>The session/quick insert session.</value>
            public Keys Session_QuickInsertPatient
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the view/navigation menu shortcut.
            /// </summary>
            /// <value>The view/navigation menu.</value>
            public Keys View_NavigationMenu
            {
                get;
                set;
            }

            #endregion Properties
        }

        #endregion Nested Types
    }
}