﻿namespace Phoenix.View.Helpers.Controls
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    /// <summary>
    /// 
    /// </summary>
    public partial class TitledDateTimePicker : UserControl, ITitledControl
    {
        #region Fields

        IList<int> hours = new List<int>();
        IList<int> minutes = new List<int>();
        private bool showTime = true;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TitledDateTimePicker"/> class.
        /// </summary>
        public TitledDateTimePicker()
        {
            InitializeComponent();

            for (int i = 0; i < 24; i++) hours.Add(i);
            for (int i = 0; i < 60; i += 15) minutes.Add(i);

            this.cb_Hours.DataSource = this.hours;
            this.cb_Minutes.DataSource = this.minutes;
            this.FullDateDisplay = false;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether to display the full date if <c>True</c> or the short one if <c>False</c>.
        /// </summary>
        /// <value><c>true</c> if [full date display]; otherwise, <c>false</c>.</value>
        public bool FullDateDisplay
        {
            get
            {
                return this.dtp_Date.Format == DateTimePickerFormat.Long;
            }
            set
            {
                this.dtp_Date.Format = (value)
                    ? DateTimePickerFormat.Long
                    : DateTimePickerFormat.Short;
            }
        }

        /// <summary>
        /// Gets or sets the width of the label's column.
        /// </summary>
        /// <value>The width of the label.</value>
        public float LabelWidth
        {
            get
            {
                return this.tableLayoutPanel.ColumnStyles[0].Width;
            }
            set
            {
                this.tableLayoutPanel.ColumnStyles[0].Width = value;
            }
        }

        /// <summary>
        /// Gets or sets the maximum date and time that can be selected in the control.
        /// </summary>
        /// <value>
        /// The maximum date and time that can be selected in the control. The default
        /// is 12/31/9998 23:59:59.</value>
        /// <exception cref="System.SystemException">
        /// The value assigned is more than the System.Windows.Forms.DateTimePicker.MinDateTime
        /// value.
        /// </exception>
        public DateTime MaxDate
        {
            get
            {
                return this.dtp_Date.MaxDate;
            }
            set
            {
                this.dtp_Date.MaxDate = value;
            }
        }

        /// <summary>
        /// Gets or sets the minimum date and time that can be selected in the control.
        /// </summary>
        /// <value>
        /// The minimum date and time that can be selected in the control. The default
        /// is 1/1/1753 00:00:00.</value>
        /// <exception cref="System.SystemException">
        /// The value assigned is less than the System.Windows.Forms.DateTimePicker.MinDateTime
        /// value.
        /// </exception>
        public DateTime MinDate
        {
            get
            {
                return this.dtp_Date.MinDate;
            }
            set
            {
                this.dtp_Date.MinDate = value;
            }
        }

        /// <summary>
        /// Gets or sets the selected item.
        /// </summary>
        /// <value>The selected item.</value>
        public object SelectedItem
        {
            get
            {
                if (this.DesignMode) return DateTime.Now;

                var date = this.dtp_Date.Value;

                return new DateTime(date.Year, date.Month, date.Day
                    , (int)cb_Hours.SelectedItem, (int)cb_Minutes.SelectedItem, 0, 0);
            }
            set
            {
                if (value is DateTime)
                {
                    var dateTime = (DateTime)value;
                    this.dtp_Date.Value = dateTime;
                    this.cb_Hours.SelectedItem = dateTime.Hour;
                    this.cb_Minutes.SelectedItem = dateTime.Minute;
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to show time input.
        /// </summary>
        /// <value><c>true</c> if [show time]; otherwise, <c>false</c>.</value>
        public bool ShowTime
        {
            get
            {
                return this.showTime;
            }
            set
            {
                this.showTime = value;
                this.DisplayTime(value);
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get
            {
                return this.label.Text;
            }
            set
            {
                this.label.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        /// <value>The value.</value>
        public DateTime Value
        {
            get
            {
                return (DateTime)this.SelectedItem;
            }
            set
            {
                this.SelectedItem = value;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds a new data binding.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="dataMember">The data member.</param>
        public void AddDataBinding(object dataSource, string dataMember)
        {
            this.dtp_Date.DataBindings.Add("Value", dataSource, dataMember);
        }

        /// <summary>
        /// Clears the data binding.
        /// </summary>
        public void ClearDataBinding()
        {
            this.dtp_Date.DataBindings.Clear();
        }

        private void DisplayTime(bool value)
        {
            this.cb_Hours.Visible
                = this.cb_Minutes.Visible
                = this.lbl_Separator.Visible
                = value;

            var columnSpan = (value) ? 1 : 4;
            this.tableLayoutPanel.SetColumnSpan(this.dtp_Date, columnSpan);
            this.dtp_Date.Dock = DockStyle.Fill;
        }

        #endregion Methods
    }
}