﻿namespace Phoenix.View.Helpers
{
    using System;
    using System.Diagnostics;

    using MigraDoc.DocumentObjectModel;
    using MigraDoc.DocumentObjectModel.Tables;
    using MigraDoc.Rendering;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter;
    using Phoenix.Translation;
    using Phoenix.View.Configuration;
    using Phoenix.View.Properties;
    using Phoenix.Helpers.Winforms;
    using log4net;

    /// <summary>
    /// Create a pdf file for a prescription
    /// </summary>
    public class PrescriptionPdf
    {
        #region Fields

        private IniFile nini;
        private log4net.ILog logger = LogManager.GetLogger(typeof(PrescriptionPdf));
        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PrescriptionPdf"/> class with
        /// the configuration of the page of the pdf.
        /// </summary>
        public PrescriptionPdf()
        {
            this.PageSetup = new PageSetup();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the file path.
        /// </summary>
        /// <value>The file path.</value>
        private string FilePath
        {
            get
            {
                return Settings.Default.PrescriptionPdf;
            }
        }

        private PageSetup PageSetup
        {
            get;
            set;
        }

        private Patient Patient
        {
            get;
            set;
        }

        private Prescription Prescription
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Builds the specified prescription for the specified patient into a PDF.
        /// </summary>
        /// <param name="patient">The patient.</param>
        /// <param name="prescription">The prescription.</param>
        public void BuildPdf(Patient patient, Prescription prescription)
        {
            this.SetupPage();
            this.Prescription = prescription;
            this.Patient = patient;

            Document document = this.Create();

            PdfDocumentRenderer renderer = new PdfDocumentRenderer(true, PdfSharp.Pdf.PdfFontEmbedding.Always);
            renderer.Document = document;
            renderer.RenderDocument();

            renderer.Save(this.FilePath);
            try
            {
                Process.Start(this.FilePath);
            }
            catch (Exception ex)
            {
                MessageBoxes.Warning("Impossible to display the pdf: " + ex.Message);
                this.logger.Warn("Impossible to display the pdf", ex);
            }
        }

        /// <summary>
        /// Defines the styles used in the document.
        /// </summary>
        public void DefineStyles(Document document)
        {
            // Get the predefined style Normal.
            Style style = document.Styles["Normal"];
            // Because all styles are derived from Normal, the next line changes the
            // font of the whole document. Or, more exactly, it changes the font of
            // all styles and paragraphs that do not redefine the font.
            style.Font.Name = "Times New Roman";

            // Heading1 to Heading9 are predefined styles with an outline level. An outline level
            // other than OutlineLevel.BodyText automatically creates the outline (or bookmarks)
            // in PDF.

            style = document.Styles["Heading1"];
            style.Font.Name = "Tahoma";
            style.Font.Size = 14;
            style.Font.Bold = true;
            style.Font.Color = Colors.DarkBlue;
            style.ParagraphFormat.PageBreakBefore = false;
            style.ParagraphFormat.SpaceAfter = 6;

            style = document.Styles["Heading2"];
            style.Font.Size = 12;
            style.Font.Bold = true;
            style.ParagraphFormat.PageBreakBefore = false;
            style.ParagraphFormat.SpaceBefore = 6;
            style.ParagraphFormat.SpaceAfter = 6;

            style = document.Styles["Heading3"];
            style.Font.Size = 10;
            style.Font.Bold = true;
            style.Font.Italic = true;
            style.ParagraphFormat.SpaceBefore = 6;
            style.ParagraphFormat.SpaceAfter = 3;

            style = document.Styles[StyleNames.Header];
            style.ParagraphFormat.AddTabStop("16cm", TabAlignment.Right);

            style = document.Styles[StyleNames.Footer];
            style.ParagraphFormat.AddTabStop("8cm", TabAlignment.Center);

            // Create a new style called TextBox based on style Normal
            style = document.Styles.AddStyle("TextBox", "Normal");
            style.ParagraphFormat.Alignment = ParagraphAlignment.Justify;
            style.ParagraphFormat.Borders.Width = 2.5;
            style.ParagraphFormat.Borders.Distance = "3pt";
            style.ParagraphFormat.Shading.Color = Colors.SkyBlue;

            // Create a new style called TOC based on style Normal
            style = document.Styles.AddStyle("TOC", "Normal");
            style.ParagraphFormat.AddTabStop("16cm", TabAlignment.Right, TabLeader.Dots);
            style.ParagraphFormat.Font.Color = Colors.Blue;
        }

        private Document Create()
        {
            Document result = new Document();
            result.Info.Title = "Prescription";
            result.Info.Subject = string.Format("Prescription of {0}", DateTime.Today.ToShortDateString());
            result.Info.Author = AppContext.CurrentUser.ToString();

            this.DefineStyles(result);
            this.Fill(result);

            return result;
        }

        private void Fill(Document document)
        {
            this.SetupPage(document);
            document.AddSection();
            Paragraph p = document.LastSection.AddParagraph();
            p.Format.Alignment = ParagraphAlignment.Right;

            Address a = AppContext.CurrentUser.MainPractice.Address;
            p.AddText(string.Format("{1}{0}{2}{0}{3}{0}{4}"
            , Environment.NewLine
            , AppContext.CurrentUser.ToString()
            , AppContext.CurrentUser.Header
            , a.Street + " " + a.StreetNumber
            , a.PostalCode + " " + a.City));

            p = document.LastSection.AddParagraph("Prescription".Translate(), "Heading1");
            this.FillDrugs(document);

            p = document.LastSection.AddParagraph("Remark".Translate(), "Heading1");
            p = document.LastSection.AddParagraph(this.Prescription.Remark ?? "");
        }

        private void FillDrugs(Document document)
        {
            Table table = new Table();
            table.Borders.Width = 0.75;

            Column column = table.AddColumn(Unit.FromCentimeter(2));
            column.Format.Alignment = ParagraphAlignment.Center;
            table.AddColumn(Unit.FromCentimeter(6));

            Row row = table.AddRow();
            row.Shading.Color = Colors.LightGray;
            row.Cells[0].AddParagraph("Name".Translate());
            row.Cells[1].AddParagraph("Type".Translate());

            foreach (var drug in this.Prescription.Drugs)
            {
                row = table.AddRow();
                row.Cells[0].AddParagraph(drug.Title.Translate());
                row.Cells[1].AddParagraph(drug.Type.Title.Translate());
            }

            table.SetEdge(0, 0, 2, Prescription.Drugs.Count - 1, Edge.Box, BorderStyle.Single, 1.5, Colors.Black);
            document.LastSection.Add(table);
        }

        private Unit Get(string key)
        {
            var value = double.Parse(nini.Get(key));
            return new Unit(value, UnitType.Millimeter);
        }

        private void SetupPage()
        {
            try
            {
                nini = new IniFile("Prescription_PageSetup");
                this.PageSetup.PageHeight = this.Get("PageHeight");
                this.PageSetup.PageWidth = this.Get("PageWidth");
                this.PageSetup.LeftMargin = this.Get("LeftMargin");
                this.PageSetup.RightMargin = this.Get("RightMargin");
                this.PageSetup.BottomMargin = this.Get("BottomMargin");
                this.PageSetup.TopMargin = this.Get("TopMargin");
            }
            catch (Exception ex)
            {
                throw new Exception("Impossible to setup the page to create the prescription: "
                    + ex.Message
                    , ex); ;
            }
        }

        private void SetupPage(Document document)
        {
            document.DefaultPageSetup.PageHeight = this.PageSetup.PageHeight;
            document.DefaultPageSetup.PageWidth = this.PageSetup.PageWidth;
            document.DefaultPageSetup.LeftMargin = this.PageSetup.LeftMargin;
            document.DefaultPageSetup.RightMargin = this.PageSetup.RightMargin;
            document.DefaultPageSetup.BottomMargin = this.PageSetup.BottomMargin;
            document.DefaultPageSetup.TopMargin = this.PageSetup.TopMargin;
        }

        #endregion Methods
    }
}