﻿namespace Phoenix.View.Input
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Translation;

    /// <summary>
    /// This manager is the basis to make a manager to manager
    /// domain item with two fields for instance name and remark.
    /// </summary>
    public partial class BasicInput : TranslateableForm
    {
        #region Fields

        private Usage usage = Usage.Display;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BasicInput"/> class.
        /// </summary>
        /// <param name="itemToManage">A default instance for item to manage.</param>
        /// <param name="dataMember1">The data member1 is the name of the property of the specified itemToManage to link with the GUI.</param>
        /// <param name="dataMember2">The data member2 is the name of the property of the specified itemToManage to link with the GUI.</param>
        protected BasicInput(object itemToManage, string dataMember1, string dataMember2)
        {
            this.Logger = LogManager.GetLogger(typeof(BasicInput));
            InitializeComponent();
            this.Usage = Usage.Insert;
            this.Text1 = "Title:".Translate();
            this.Text2 = "Remark".Translate();
            this.Bind(itemToManage, dataMember1, dataMember2);
        }

        #endregion Constructors

        #region Enumerations

        /// <summary>
        /// Indicates on which control the error should be displayed
        /// </summary>
        protected enum On
        {
            /// <summary>
            /// The error is displayed on the TextBox for the Value1.
            /// </summary>
            Value1,
            /// <summary>
            /// The error is displayed on the TextBox for the Value2.
            /// </summary>
            Value2,
            /// <summary>
            /// The error is displayed on the TextBox for both the Value1 and the Value2.
            /// </summary>
            Both
        }

        #endregion Enumerations

        #region Events

        /// <summary>
        /// Occurs when user asked to delete an item
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when user asked to insert an item
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when user asked to update an item
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the value of the first label.
        /// </summary>
        /// <value>The text of the label.</value>
        public string Text1
        {
            get
            {
                return this.lbl_Value1.Text;
            }
            set
            {
                this.lbl_Value1.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the of the second label.
        /// </summary>
        /// <value>The text of the label.</value>
        public string Text2
        {
            get
            {
                return this.lbl_Value2.Text;
            }
            set
            {
                this.lbl_Value2.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the text associated with this control..
        /// </summary>
        /// <value>The text associated with this control..</value>
        public string Title
        {
            get { return this.Text; }
            set { this.Text = value; }
        }

        /// <summary>
        /// Gets or sets the usage of this manager.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get { return this.usage; }
            set
            {
                this.usage = value;
                switch (value)
                {
                    case Usage.Insert:
                        this.btn_Add.Text = "Add".Translate();
                        break;
                    case Usage.Update:
                        this.btn_Add.Text = "Update".Translate();
                        break;
                    case Usage.Delete:
                        this.btn_Add.Text = "Delete".Translate();
                        this.tb_Value1.ReadOnly
                            = this.tb_Value2.ReadOnly
                            = true;
                        break;
                    default:
                        this.Logger.WarnFormat("Unexpected usage:'{0}'", value);
                        break;
                }
            }
        }

        /// <summary>
        /// Gets yhe item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        protected object Item
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Binds the specified item to manage into the two textboxes.
        /// The datamembers will link the textbox with the member of the specified
        /// itemToManage that has the specified name
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        /// <param name="dataMember1">The data member1.</param>
        /// <param name="dataMember2">The data member2.</param>
        public void Bind(object itemToManage, string dataMember1, string dataMember2)
        {
            string propertyName = "Text";
            this.Item = itemToManage;

            this.tb_Value1.DataBindings.Clear();
            this.tb_Value2.DataBindings.Clear();

            this.tb_Value1.DataBindings.Add(propertyName, this.Item, dataMember1);
            this.tb_Value2.DataBindings.Add(propertyName, this.Item, dataMember2);
        }

        /// <summary>
        /// Clears the errors.
        /// </summary>
        public void ClearError()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Deletes the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected virtual void Delete(object itemToManage)
        {
            this.Deleted.Raise(this);
        }

        /// <summary>
        /// Inserts the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected virtual void Insert(object itemToManage)
        {
            this.Inserted.Raise(this);
        }

        /// <summary>
        /// Sets the error on value1.
        /// </summary>
        /// <param name="onValue">The on value.</param>
        /// <param name="message">The message.</param>
        protected void SetError(On onValue, string message)
        {
            List<Control> controls = new List<Control>();
            switch (onValue)
            {
                case On.Value1:
                    controls.Add(this.tb_Value1);
                    break;
                case On.Value2:
                    controls.Add(this.tb_Value2);
                    break;
                case On.Both:
                    controls.Add(this.tb_Value1);
                    controls.Add(this.tb_Value2);
                    break;
                default:
                    Assert.FailsOnEnumeration(onValue);
                    break;
            }

            foreach (var control in controls)
            {
                this.ep_Error.SetError(control, message);
            }
        }

        /// <summary>
        /// Updates the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected virtual void Update(object itemToManage)
        {
            this.Updated.Raise(this);
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>Returns <c>true</c> if the data is valid; otherwise <b>false</b></returns>
        protected virtual bool ValidateData()
        {
            return true;
        }

        private void BasicManager_Load(object sender, EventArgs e)
        {
            this.tb_Value1.Select();
        }

        private void btn_Add_Click(object sender, EventArgs e)
        {
            switch (this.Usage)
            {
                case Usage.Insert:
                    if (this.ValidateData())
                    {
                        this.Insert(this.Item);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Update:
                    if (this.ValidateData())
                    {
                        this.Update(this.Item);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Delete:
                    if (this.ValidateData())
                    {
                        this.Delete(this.Item);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                default:
                    Assert.FailsOnEnumeration(this.Usage);
                    break;
            }
        }

        private void btn_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        #endregion Methods
    }
}