﻿namespace Phoenix.Presenter.Management.Controls
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter.Interfaces.Input;

    /// <summary>
    /// Control that provides possibility to manage a doctor.
    /// </summary>
    public partial class DoctorManagerControl : UserControl
    {
        #region Fields

        private readonly List<DoctorType> specialisation = new List<DoctorType>();

        private Doctor doctor = new Doctor();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DoctorManagerControl"/> class.
        /// </summary>
        public DoctorManagerControl()
        {
            this.Logger = LogManager.GetLogger(typeof(DoctorManagerControl));
            InitializeComponent();
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user is asking to add a new specialisation.
        /// </summary>
        public event EventHandler AddingSpecialisation;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the item this manager manages.
        /// </summary>
        /// <value>The managed item.</value>
        public Doctor Doctor
        {
            get
            {
                this.doctor.Address = this.ac_Address.Address;
                this.doctor.FirstName = this.tb_FirstName.Text;
                this.doctor.IsComplete = true;
                this.doctor.LastName = this.tb_LastName.Text;
                this.doctor.LastUpdate = DateTime.Now;
                this.doctor.Mail = this.tb_Email.Text;
                this.doctor.MobilePro = this.tb_MobilePro.Text;
                this.doctor.PhonePro = this.tb_PhonePro.Text;
                this.doctor.Sex = (this.rb_Male.Checked) ? "M" : "F";

                if (!this.DesignMode)
                {
                    DoctorType type = cb_Specialisation.SelectedItem as DoctorType;
                    this.doctor.Specialisation = type;
                }

                this.doctor.Thumbnail = pix_Picture.Image.ToBytes();

                return this.doctor;
            }
            set
            {
                if (value == null)
                {
                    var ex = new NullReferenceException("Null patient inserted.");
                    this.Logger.Warn(ex);
                }

                this.doctor = value;
                this.ac_Address.Address = value.Address;
                this.tb_FirstName.Text = value.FirstName;
                this.tb_LastName.Text = value.LastName;
                this.tb_Email.Text = value.Mail;
                this.tb_MobilePro.Text = value.MobilePro;
                this.tb_PhonePro.Text = value.PhonePro;
                this.cb_Specialisation.SelectedItem = value.Specialisation;

                if (value.Specialisation == null)
                    this.Logger.DebugFormat("The doctor '{0}' doesn't have specialisation (value = null)"
                        , value.ID);

            }
        }

        /// <summary>
        /// Gets or sets the list of all the specialisations and link it to the combobox.
        /// </summary>
        /// <value>The specialisations.</value>
        public List<DoctorType> Specialisations
        {
            get
            {
                return this.specialisation;
            }
            set
            {
                this.specialisation.Clear();
                this.specialisation.AddRange(value);
                this.cb_Specialisation.DataSource = null;
                this.cb_Specialisation.DataSource = value;
            }
        }

        private ILog Logger
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        public void ClearErrors()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="fail">The fail.</param>
        /// <param name="errorMessage">The message.</param>
        public void SetError(DoctorFailOn fail, string errorMessage)
        {
            Control control = null;
            switch (fail)
            {
                case DoctorFailOn.LastNameIsNum:
                    control = this.tb_LastName;
                    break;
                case DoctorFailOn.LastNameIsEmpty:
                    control = this.tb_LastName;
                    break;
                case DoctorFailOn.FirstNameIsNum:
                    control = this.tb_FirstName;
                    break;
                case DoctorFailOn.FirstNameIsEmpty:
                    control = this.tb_FirstName;
                    break;
                case DoctorFailOn.EmailIsNotValid:
                    control = this.tb_Email;
                    break;
                case DoctorFailOn.ProPhoneIsNotValid:
                    control = this.tb_PhonePro;
                    break;
                case DoctorFailOn.ProMobileIsNotValid:
                    control = this.tb_MobilePro;
                    break;
                case DoctorFailOn.NoSpecialisation:
                    control = this.cb_Specialisation;
                    break;
                case DoctorFailOn.StreetIsNum:
                    this.ac_Address.InvalidateStreet(errorMessage);
                    break;
                case DoctorFailOn.CityIsNum:
                    this.ac_Address.InvalidateCity(errorMessage);
                    break;
                default:
                    Assert.FailsOnEnumeration(fail);
                    break;
            }
            if (control != null)
                this.ep_Error.SetError(control, errorMessage);
        }

        private void llb_AddPicture_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            DialogResult dr = ofd_Thumbnail.ShowDialog();
            if (dr != DialogResult.OK) return;

            string fileName = ofd_Thumbnail.FileName;
            try
            {
                Image image = Image.FromFile(fileName);
                Image thumb = image.GetThumbnailImage(100, 100);

                this.pix_Picture.Image = thumb;
            }
            catch (OutOfMemoryException)
            {
                MessageBoxes.Exclamation("Please select a valid image!");
            }
        }

        private void llb_AddSpec_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            this.AddingSpecialisation.Raise(this);
        }

        #endregion Methods
    }
}