﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Translation;

    /// <summary>
    /// Gui to manage the family of the patient
    /// </summary>
    public partial class FamilyManagerBox : TranslateableForm, IFamilyManagerPresenter
    {
        #region Fields

        private const string EXPLANATION = @"Here is the family of {0}.  Right click in the table to add/modify/remove an entry.
         Double click on a member of the family to open a session for him/her";
        private const string TITLE = "Family of {0}";

        private FamilyModel model;
        private Patient patient;
        private List<RelationType> relationTypes = new List<RelationType>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="FamilyManagerBox"/> class.
        /// </summary>
        public FamilyManagerBox()
        {
            InitializeComponent();
            this.model = new FamilyModel(this);
            AppContext.DataChanged += AppContext_DataChanged;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a patient is loading.
        /// </summary>
        public event EventHandler<EventArgs<Patient>> PatientLoading;

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when a relation is deleted.
        /// </summary>
        public event EventHandler<EventArgs<Relation>> RelationDeleted;

        /// <summary>
        /// Occurs when a relation is inserted.
        /// </summary>
        public event EventHandler<EventArgs<Relation>> RelationInserted;

        /// <summary>
        /// Occurs when a relation is updated.
        /// </summary>
        public event EventHandler<EventArgs<Relation>> RelationUpdated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Family".Translate(); }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return Phoenix.Presenter.Interfaces.IconValue.Family; }
        }

        /// <summary>
        /// Gets or sets the patient.
        /// </summary>
        /// <value>The patient.</value>
        public Patient Patient
        {
            get
            {
                return this.patient;
            }
            set
            {
                this.patient = value;
                this.Text = TITLE.Translate(value.ToString());
                this.lbl_Explanation.Text = EXPLANATION.Translate(value.ToString());
            }
        }

        /// <summary>
        /// Gets or sets the relation types.
        /// </summary>
        /// <value>The relation types.</value>
        public List<RelationType> RelationTypes
        {
            get
            {
                return this.relationTypes;
            }
            set
            {
                this.relationTypes = value;

                this.lv_Family.Groups.Clear();
                foreach (var type in this.relationTypes)
                {
                    this.lv_Family.Groups.Add(new ListViewGroup(type.Title, type.Title));
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        public void RefreshGui()
        {
            this.lv_Family.Items.Clear();
            foreach (var relation in this.Patient.FamilyMembers)
            {
                ListViewItem item = new ListViewItem(relation.Member.ToString());
                item.Group = lv_Family.Groups[relation.Type.Title];
                item.ImageKey = (relation.Member.Sex == "M") ? "memberM" : "memberF";
                item.Tag = relation;

                this.lv_Family.Items.Add(item);
            }
        }

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="failure"></param>
        /// <param name="errorMessage"></param>
        public void SetError(FamilyFailedOn failure, string errorMessage)
        {
            switch (failure)
            {
                case FamilyFailedOn.RecursiveRelation:
                    MessageBoxes.Exclamation(errorMessage);
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            AppContext.DataChanged -= AppContext_DataChanged;
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void AppContext_DataChanged(object sender, EventArgs e)
        {
            this.RefreshData();
        }

        private void cms_Context_Opening(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (lv_Family.SelectedItems.Count == 0)
            {
                this.tsmi_Delete.Enabled = false;
                this.tsmi_Update.Enabled = false;
            }
            else
            {
                this.tsmi_Delete.Enabled = true;
                this.tsmi_Update.Enabled = true;
            }
        }

        private void FamilyBox_Load(object sender, System.EventArgs e)
        {
            this.RefreshData();
        }

        private void lv_Family_ItemActivate(object sender, System.EventArgs e)
        {
            if (this.lv_Family.SelectedItems.Count <= 0) return;

            Relation relation = this.lv_Family.SelectedItems[0].Tag as Relation;
            this.PatientLoading.Raise(this, new EventArgs<Patient>(relation.Member));
        }

        private void OnRelationDeleted(Relation relation)
        {
            Assert.IsNotNull(relation, "relation");
            this.RelationDeleted.Raise(this, new EventArgs<Relation>(relation));
        }

        private void OnRelationInserted(Relation relation)
        {
            if (this.model.ValidateData(relation))
            {
                this.RelationInserted.Raise(this, new EventArgs<Relation>(relation));
            }
        }

        private void OnRelationUpdated(Relation relation)
        {
            if (this.model.ValidateData(relation))
            {
                this.RelationUpdated.Raise(this, new EventArgs<Relation>(relation));
            }
        }

        private void tsmi_Add_Click(object sender, System.EventArgs e)
        {
            IRelationInputPresenter box = GuiManager.Get<IRelationInputPresenter>();
            box.Patient = this.Patient;
            DialogResult dr = GuiManager.ShowDialog(box);

            if (dr != DialogResult.OK) return;
            this.OnRelationInserted(box.SelectedRelation);
            this.RefreshData();
        }

        private void tsmi_Delete_Click(object sender, EventArgs e)
        {
            if (this.lv_Family.SelectedItems.Count < 00) return;
            DialogResult dr = MessageBoxes.Question("Do you want to delete this relation?".Translate());
            if (dr == DialogResult.No) return;

            this.OnRelationDeleted(this.lv_Family.SelectedItems[0].Tag as Relation);
            this.RefreshData();
        }

        private void tsmi_Update_Click(object sender, EventArgs e)
        {
            if (this.lv_Family.SelectedItems.Count < 00) return;

            Relation relation = this.lv_Family.SelectedItems[0].Tag as Relation;
            IRelationInputPresenter box = GuiManager.Get<IRelationInputPresenter>();
            box.Patient = patient;
            box.SelectedRelation = this.lv_Family.SelectedItems[0].Tag as Relation;
            box.Usage = Usage.Update;

            DialogResult dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;

            this.OnRelationUpdated(box.SelectedRelation);
            this.RefreshData();
        }

        #endregion Methods
    }
}