﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Presenter.Interfaces.Interfaces;

    /// <summary>
    /// Displays patient
    /// </summary>
    public partial class PatientDisplayer : TranslateableForm, IPatientManagerPresenter
    {
        #region Fields

        private bool backgroundWork = false;
        private IList<Insurance> insurances = new List<Insurance>();
        private PatientDisplayerModel model;
        private Patient patient;
        private IList<Practice> practices = new List<Practice>();
        private IList<Reputation> reputations = new List<Reputation>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PatientDisplayer"/> class.
        /// </summary>
        public PatientDisplayer()
        {
            InitializeComponent();
            this.model = new PatientDisplayerModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a patient is loaded.
        /// </summary>
        public event EventHandler PatientLoaded;

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when the patient is updated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether a background thread is working and this deactivate
        /// the Gui.
        /// </summary>
        /// <value><c>true</c> if [background work]; otherwise, <c>false</c>.</value>
        public bool BackgroundWork
        {
            get
            {
                return this.backgroundWork;
            }
            set
            {
                this.backgroundWork = value;
                if (value)
                {
                    this.EnableGuiItems(false);
                    Cursor.Current = Cursors.AppStarting;
                }
                else
                {
                    this.EnableGuiItems(true);
                    Cursor.Current = Cursors.Arrow;
                }
            }
        }

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Information"; }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.Patient; }
        }

        /// <summary>
        /// Gets or sets the insurances.
        /// </summary>
        /// <value>The insurances.</value>
        public IList<Insurance> Insurances
        {
            get
            {
                return this.insurances;
            }
            set
            {
                this.insurances = value;
                this.tcb_Insurance.Bind(value, "Title");
            }
        }

        /// <summary>
        /// Gets or sets the patient.
        /// </summary>
        /// <value>The patient.</value>
        public Patient Patient
        {
            get
            {
                return this.patient;
            }
            set
            {
                this.patient = value;
                this.Binding();
            }
        }

        /// <summary>
        /// Gets or sets the practices.
        /// </summary>
        /// <value>The practices.</value>
        public IList<Practice> Practices
        {
            get
            {
                return this.practices;
            }
            set
            {
                this.practices = value;
                this.tcb_Practice.Bind(value, "Title");
            }
        }

        /// <summary>
        /// Gets or sets the reputations.
        /// </summary>
        /// <value>The reputations.</value>
        public IList<Reputation> Reputations
        {
            get
            {
                return this.reputations;
            }
            set
            {
                this.reputations = value;
                this.tcb_Reputation.Bind(value, "Title");
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when a patient is loaded.
        /// </summary>
        public void OnPatientLoaded()
        {
            if (this.PatientLoaded != null) this.PatientLoaded(this, EventArgs.Empty);
        }

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data of the gui.
        /// </summary>
        public void RefreshGui()
        {
            this.s_Title.Title = string.Format("{0} {1} [{2} ans]"
                , this.Patient.FirstName
                , this.Patient.LastName.ToUpper()
                , this.Patient.Age);

            this.Text = string.Format("Info of {0} {1}"
                , this.patient.FirstName
                , this.patient.LastName);

            //Miscellaneous
            if (this.patient.Insurance != null) this.tcb_Insurance.SelectText(this.patient.Insurance.Title);
            else this.tcb_Insurance.SelectedIndex = -1;

            if (this.patient.Practice != null) this.tcb_Practice.SelectText(this.patient.Practice.Title);
            else this.tcb_Practice.SelectedIndex = -1;

            if (this.patient.Reputation != null) this.tcb_Reputation.SelectText(this.patient.Reputation.Title);
            else this.tcb_Reputation.SelectedIndex = -1;

            //Doctors
            this.lv_Doctors.Items.Clear();
            foreach (var doctor in this.patient.Doctors) this.AddInListView(doctor);

            //Databindings problem with these items
            this.tdtp_Dob.Value = this.patient.DateOfBirth;
            this.tdtp_LastUpdate.Value = this.patient.LastUpdate;
            this.tdtp_Inscription.Value = this.patient.InscriptionDate;
            this.tdtp_LastUpdate.Value = this.patient.LastUpdate;
            this.tyn_Gender.YesValue = this.patient.IsMale;
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void AddInListView(Doctor doctor)
        {
            var name = string.Format("{0} {1}", doctor.FirstName, doctor.LastName.ToUpper());

            var item = new ListViewItem(doctor.Specialisation.Title);
            item.ImageKey = "doctor";
            item.Tag = doctor;
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, name));

            this.lv_Doctors.Items.Add(item);
        }

        private void Binding()
        {
            this.RefreshGui();

            //Validation
            this.tdtp_LastUpdate.MaxDate
                = this.tdtp_Inscription.MaxDate
                = this.tdtp_Dob.MaxDate
                = DateTime.Today.AddDays(1);

            //Personal information
            this.tpi_Picture.Image = this.patient.ThumbnailImage;
            this.ttb_LastName.AddDataBinding(this.patient, "LastName");
            this.ttb_FirstName.AddDataBinding(this.Patient, "FirstName");
            //this.tdtp_Dob.AddDataBinding(this.Patient, "DateOfBirth");
            this.ttb_Pob.AddDataBinding(this.patient, "PlaceOfBirth");
            //this.tyn_Gender.AddDataBinding(this.patient, "IsMale");

            //Contact information
            this.ttb_Email.AddDataBinding(this.patient, "Mail");
            this.ttb_PhonePro.AddDataBinding(this.patient, "PhonePro");
            this.ttb_MobilePro.AddDataBinding(this.patient, "MobilePro");
            this.ttb_PhonePrivate.AddDataBinding(this.patient, "PhonePrivate");
            this.ttb_MobilePrivate.AddDataBinding(this.patient, "MobilePrivate");

            this.cb_IsComplete.Checked = this.patient.IsComplete;

            //Address information
            if (this.Patient.Address == null) this.patient.Address = new Address();
            this.ttb_Street.AddDataBinding(this.patient.Address, "Street");
            this.ttb_StreetNumber.AddDataBinding(this.patient.Address, "StreetNumber");
            this.ttb_BoxNr.AddDataBinding(this.patient.Address, "BoxNumber");
            this.ttb_City.AddDataBinding(this.patient.Address, "City");
            this.ttb_PostalCode.AddDataBinding(this.patient.Address, "PostalCode");

            //Reason
            this.tml_Reason.AddDataBinding(this.patient, "Reason");
            //this.tdtp_LastUpdate.AddDataBinding(this.patient, "LastUpdate");
        }

        private void cms_Doctors_Opening(object sender, CancelEventArgs e)
        {
            this.tsmi_removeDoctor.Visible = (this.lv_Doctors.SelectedItems.Count > 0);
        }

        private void EnableGuiItems(bool value)
        {
            this.tsb_Save.Enabled
                = this.tcb_Insurance.Enabled
                = this.tcb_Practice.Enabled
                = this.tcb_Reputation.Enabled
                = value;
        }

        private void flp_Resize(object sender, EventArgs e)
        {
            if (sender is Panel)
            {
                var parent = sender as Panel;
                foreach (Control control in parent.Controls) control.Width = parent.Width - 6;
            }
        }

        private void OnUpdated()
        {
            this.Updated.Raise(this);
            AppContext.NotifySessionUpdated(this.patient);
        }

        private void PatientDisplayer_Load(object sender, EventArgs e)
        {
            /*Deactivate picture modification on XP, and below while I'm inverstigating on why the application crashes*/
            this.tpi_Picture.CanChangeImage = !(OS.IsWindows2000 || OS.IsWindows2000 || OS.IsWindowsXP || OS.IsWindowsXP64K);

            this.Refreshed.Raise(this);
        }

        private void tsb_Save_Click(object sender, EventArgs e)
        {
            if (tcb_Insurance.SelectedItem is Insurance) this.patient.Insurance = tcb_Insurance.SelectedItem as Insurance;
            if (tcb_Reputation.SelectedItem is Reputation) this.patient.Reputation = tcb_Reputation.SelectedItem as Reputation;
            if (tcb_Practice.SelectedItem is Practice) this.patient.Practice = tcb_Practice.SelectedItem as Practice;

            this.patient.IsComplete = this.cb_IsComplete.Checked;

            /* The data binding doesn't work fine if I want to refresh
               the side menu. This is a temporary solution before
               I find the real solution.*/
            this.patient.FirstName = this.ttb_FirstName.Value;
            this.patient.LastName = this.ttb_LastName.Value;
            this.patient.Thumbnail = this.tpi_Picture.Image.ToBytes();
            this.patient.DateOfBirth = this.tdtp_Dob.Value;
            this.patient.LastUpdate = this.tdtp_LastUpdate.Value;
            this.patient.InscriptionDate = this.tdtp_Inscription.Value;
            this.patient.LastUpdate = DateTime.Now;
            this.patient.IsMale = tyn_Gender.YesValue;

            this.OnUpdated();
        }

        private void tsmi_AddDoctor_Click(object sender, EventArgs e)
        {
            var form = GuiManager.Get<ISearchBox<Doctor>>();
            GuiManager.ShowDialog(form);
            if (form.SelectedItem == null) return;
            if (this.model.CanAddDoctor(form.SelectedItem as Doctor))
            {
                this.Patient.Add(form.SelectedItem as Doctor);
                this.OnUpdated();
            }
            else
            {
                MessageBoxes.Information("You can't add twice the same doctor.");
            }
        }

        private void tsmi_displayDoctor_Click(object sender, EventArgs e)
        {
            var form = GuiManager.Get<IDoctorInputPresenter>();
            form.ItemToManage = this.lv_Doctors.SelectedItems[0].Tag as Doctor;
            GuiManager.ShowDialog(form);
        }

        private void tsmi_removeDoctor_Click(object sender, EventArgs e)
        {
            var dr = MessageBoxes.Question("Do you want to remove this doctor from the list?");
            if (dr != DialogResult.Yes) return;

            var doctor = this.lv_Doctors.SelectedItems[0].Tag as Doctor;
            Assert.IsNotNull(doctor, "doctor", "The selected doctor doesn't contain valid data");
            this.patient.Remove(doctor);
            this.OnUpdated();
        }

        #endregion Methods
    }
}