﻿namespace Phoenix.Presenter.Search.Criterion
{
    using System;
    using System.ComponentModel;
    using System.Windows.Forms;

    using Phoenix.Translation;

    /// <summary>
    /// Provides an input for numeric criterion.
    /// </summary>
    public class DateTimeCriterion : CriterionControl, ICriterionControl
    {
        #region Fields

        private DateTimePicker dtp_DateTime = new DateTimePicker();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DateTimeCriterion"/> class.
        /// </summary>
        public DateTimeCriterion()
            : this(new DateTime(1850, 1, 1))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DateTimeCriterion"/> class.
        /// </summary>
        /// <param name="minDate">The min date whereon a search can be done.</param>
        public DateTimeCriterion(DateTime minDate)
            : base()
        {
            this.MaxDate = DateTime.Today.AddDays(1);
            this.MinDate = minDate;
            base.AddInCriterionControl(this.dtp_DateTime);
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the minimum accepted date.
        /// </summary>
        /// <value>The min date.</value>
        public DateTime MinDate
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the criterion's selected DateTime.
        /// </summary>
        /// <value></value>
        /// <number>The criterion's number.</number>
        [Browsable(false)]
        public override object Value
        {
            get
            {
                if (this.IsActivated) return this.dtp_DateTime.Value.Date;
                else return null;
            }
        }

        /// <summary>
        /// Gets or sets the maximum accepted date.
        /// </summary>
        /// <value>The max date.</value>
        private DateTime MaxDate
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Checks the values the user entered and provides
        /// warning message if values are not correct.
        /// </summary>
        /// <returns><c>True</c> if the check succeed. Otherwise <c>False</c>.</returns>
        public override bool ValidateCriterions()
        {
            if (!base.IsActivated) { return true; }
            else
            {
                string msg = string.Empty;
                if (this.dtp_DateTime.Value < this.MinDate)
                {
                    msg = "The date should be after {0}".Translate(this.MinDate.ToShortDateString());
                    base.SetError(msg);
                    return false;
                }
                else if (this.dtp_DateTime.Value > this.MaxDate)
                {
                    msg = "The date should be before {0}".Translate(this.MaxDate.ToShortDateString());
                    base.SetError(msg);
                    return false;
                }
                else
                {
                    base.ClearError();
                    return true;
                }
            }
        }

        /// <summary>
        /// Clears the value of the criterion.
        /// </summary>
        protected override void ClearValue()
        {
            this.dtp_DateTime.Value = DateTime.Today;
        }

        #endregion Methods
    }
}